
/*

Copyright (c) 2018 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.  

Texas Instruments Incorporated grants a world-wide, royalty-free, non-exclusive license under copyrights and patents it now or hereafter owns or controls to make, have made, use, import, offer to sell and sell ("Utilize") this software subject to the terms herein.  With respect to the foregoing patent license, such license is granted  solely to the extent that any such patent is necessary to Utilize the software alone.  The patent license shall not apply to any combinations which include this software, other than combinations with devices manufactured by or for TI (“TI Devices”).  No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and reproduce this license (including the above copyright notice and the disclaimer and (if applicable) source code license limitations below) in the documentation and/or other materials provided with the distribution

Redistribution and use in binary form, without modification, are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of this software is permitted with respect to any software provided in binary form.

*	any redistribution and use are licensed by TI for use only with TI Devices.

*	Nothing shall obligate TI to provide you with source code for the software licensed and provided to you in object code.

If software source code is provided to you, modification and redistribution of the source code are permitted provided that the following conditions are met:

*	any redistribution and use of the source code, including any resulting derivative works, are licensed by TI for use only with TI Devices.

*	any redistribution and use of any object code compiled from the source code and any resulting derivative works, are licensed by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names of its suppliers may be used to endorse or promote products derived from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TI’S LICENSORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TI’S LICENSORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};


// Search registerCRCUser inside this file to see how you should
// invoke this function from the right place.
function registerCRCUser() {
    // Code to implement CRC
    // register a callback as a crc user for a particular interface, e.g., i2c
    // Case of AEVM, use gc.databind.AEVMCodec.prototype.register_crc_user instead.
    // gc.databind.AEVMCodec.prototype.register_crc_user( { ... }, ...);
    gc.databind.USB2ANY.prototype.register_crc_user( {
        // embed_crc_data is called back during read and write for inserting crc bytes
        //   crc is provided to compute crc byte(s)
        //   arg: {write, deviceAddr, payload}
        //   arg.write indicates this callback is for write (true) or read(false)
        //   arg.deviceAddr is the device address (a.k.a. slave address)
        //   arg.payload is the payload part of (u2a/aevm) packet.
        embed_crc_data: function(crc, arg) {
            // use crc.checksum(byte_array, optionalPreviousCRC) to compute crc data byte(s)
            // and prepend/insert/append crc data byte(s) into payload at desired place(s)
            
            // When writing this code, you can use console.log(arg) to see what properties are in 'arg'.

            // Case of AEVM: arg.num_read_bytes is the number of bytes to read without counting the crc part,
            // modify it to include the number of crc bytes for read.
            // Example code for AEVM
            // if (arg.write === true) {
            //     var c = crc.checksum([arg.deviceAddr]);
            //     if (arg.opcode !== undefined) c = crc.checksum([arg.opcode],c);
            //     c = crc.checksum(arg.payload, c);
            //     arg.payload.push(c);
            // }
            // return {payload: arg.payload, num_read_bytes: (arg.write===true)?0:arg.num_read_bytes+1};

            // Case of USB2ANY: arg.numBytes is number of bytes to read/write without crc part
            // return modified payload and number of bytes (data and crc parts)
            // Example code (uncommnet the code below)
            var extra = 0;
            //var workAroundBug = (arg.write===true) ? 1 : 0;
            // if (arg.write === true) {
            //    arg.payload.push(crc.checksum(arg.payload));
            //}
            // extra = 1; // number of crc bytes added to arg.payload for write
                       // number of crc bytes for read
            return {payload: arg.payload, numBytes: arg.numBytes+extra }

        }
        // verify_crc_data is called back after getting result of read request from controller for
        // verifying crc byte(s) and extracting actual data part out from payload
        //   crc is provided to compute crc byte(s)
        //   arg: {payload}
        //   arg.payload is the payload including both crc and data parts
        , verify_crc_data: function(crc, arg) {
            // Example code (uncommnet the code below)
            var d = arg.payload, valid = true;
            //    d = arg.payload.slice(0,-1); // the data without crc part
            //    c = crc.checksum(d);
            //    valid = c == arg.payload[arg.payload.length-1];
            // If you do not plan to verify the crc byte(s),
            // you still need to say valid is true, and make sure d is the
            // data without crc part. Otherwise, your app cannot properly read registers
            // and may get "connected to controller but failed to connect to device...".
            return {valid: valid, payload: d}
        }
     }, 'i2c');
    
}



var convertTemp = function(value, isToCelsius) {
    var result = value;
    if (isToCelsius) {
        result = (value - 32) / 1.8;
    } else {
        result = (value * 1.8) + 32;
    }
    return result;
};

var getF = function(value){
	return (value * 1.8) + 32;
};
var getC = function(value){
    return (value - 32) / 1.8;
};

var tempScale = "C";

var saveText = function(fname, text) {
    var blob = new Blob([text], {
        type: 'text/plain;charset=utf-8'
    });
    saveAs(blob, fname);

    return false;
};

function convertTimestamp(timestamp) {
    var d = new Date(timestamp), // Convert the passed timestamp to milliseconds
        yyyy = d.getFullYear(),
        mm = ('0' + (d.getMonth() + 1)).slice(-2), // Months are zero based. Add leading 0.
        dd = ('0' + d.getDate()).slice(-2), // Add leading 0.
        hh = d.getHours(),
        h = hh,
        min = ('0' + d.getMinutes()).slice(-2), // Add leading 0.
        ampm = 'AM',
        time;

    if (hh > 12) {
        h = hh - 12;
        ampm = 'PM';
    } else if (hh === 12) {
        h = 12;
        ampm = 'PM';
    } else if (hh === 0) {
        h = 12;
    }

    // ie: 2013-02-18, 8:35 AM	
    time = +h + ':' + min + ' ' + ampm + ',' + yyyy + '-' + mm + '-' + dd;

    return time;
}

var getHex16 = function(value) {
    // convert the register value into a human readable format for the GUI and convert to F if needed
    var result = gc.utils.getHexString(value, 16, "0x");
    return result;
};

var extendAboutBox = function() {
    var aboutBox = document.querySelector('ti-widget-aboutbox');
    if (aboutBox){
        aboutBox.addEventListener('aboutbox_opening', function(event){
            aboutBox.appInfoTextHeading = 'Details';
            aboutBox.softwareManifestLink = "app/docs/TMP117EVM GUI_1.0.16_manifest.html";
            var str = '******************************\nTMP117EVM GUI\n******************************\n';
            str = str + 'Version                 : ';
            str = str + 'Publish Date            : 06/28/2018\n';
            str = str + 'TMP117 Revision 1.0.16\n';
            str = str + '\n';
            str = str + 'Change List\n';
            str = str + '  06/28/2018: Initial Released\n';
            str = str + '\n';
            str = str + '******************************\nConnected device\n******************************\n';
            aboutBox.appInfoText = str + 'Retreiving version information ...\nPlease connect hardware';
            aboutBox.numRowsInAppInfoTextArea = 16;
        });
    }
};
/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/
	
	registerCRCUser();

	
	//Continous radio button on data page, 
	gc.databind.registry.bind('widget.ti_widget_radiobtn_continuous.checked','reg.CONFIG_REG.CONV_MODE',
        function(value) {
            // convert register value to state of a continous radio button  (i.e. checked property setting)
            var result = value;
            if ((value !== undefined) && (value !== null)) {
				if (value === 0 || value === 2) {
					result = true;
				}
				else if (value === 1 || value === 3) {
					result = false;
				}
            }
            return result;
        },
        function(value) {
            // If user enables continous mode, then write 0 to CONV_MODE bitfield
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (value) {
                     result = 0;
                }
				else {
					result = 1;
				}
            }
            return result;
        }
    );
	
	//Shutdown radio button on data page, 
	gc.databind.registry.bind('widget.ti_widget_radiobtn_shutdown.checked','reg.CONFIG_REG.CONV_MODE',
        function(value) {
            // convert register value to state of a shutdown radio button  (i.e. checked property setting)
            var result = value;
            if ((value !== undefined) && (value !== null)) {
				if (value === 1 || value === 3) {
					result = true;
					// If shutdown mode enabled, then also enable triggering of Send One Shot button widget
					templateObj.$.ti_widget_button_sendoneshot.disabled = false;
				}
				else {
					result = false;
					templateObj.$.ti_widget_button_sendoneshot.disabled = true;
				}
            }
            return result;
        },
        function(value) {
            // If user enables shutdown mode, then write 1 to CONV_MODE bitfield
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (value) {
                     result = 1;
                }
				else {
					result = 0;
				}
            }
            return result;
        }
    );
	

    gc.databind.registry.bind('widget.ti_widget_spinner_high_limit.value','reg.ALERT_HI_LIMIT.Alert_High_Limit',
        function(value) {
			if (tempScale === "C")
				return value;
			else 
				return getF(value);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );
  
   gc.databind.registry.bind('widget.ti_widget_spinner_low_limit.value','reg.ALERT_LO_LIMIT.Alert_Low_Limit',
        function(value) {
			if (tempScale === "C")
				return value;
			else 
				return getF(value);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );
    gc.databind.registry.bind('widget.ti_widget_spinner_offset_temp.value','reg.OFFSET_TEMP.OFFSETTEMP',
        function(value) {
			if (tempScale === "C")
				return value;
			else 
				return getF(value);
        },
        function(value) {
            if (tempScale === "C")
				return value;
			else 
				return getC(value);
        }
    );

	
    gc.databind.registry.bind('widget.ti_widget_label_temp.label','reg.TEMP_RESULT.TEMP',
        function(value) {
            if (tempScale === "C")
				return Number.parseFloat(value).toFixed(2);
			else 
				return Number.parseFloat(getF(value)).toFixed(2);
        }
    );
	

	/*gc.databind.registry.bind('widget.ti_widget_temperaturegraph.value','reg.TEMP_RESULT',
        function(value) {
			var temp_result =0;
			temp_result = twosComp(value);
            return temp_result;
        },
        function(value) {
			var byte_combine_result = 0;
            byte_combine_result = setDataFormat(value);
            return byte_combine_result;
        }
    );*/
    /*
    gc.databind.registry.bind('widget.ti_widget_spinner_low_limit.value', 'reg.ALERT_LO_LIMIT', 
        function(value) {
            // convert the register value into a human readable format for the GUI and convert to F if needed
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (tempScale !== 'C') {
                     result = getF(value);
                }
            }
            return result;
        },
        function(value) {
            // convert the value entered in the GUI into Celsius, the format needed by the TMP117 register
            var result = value;
            if ((value !== undefined) && (value !== null)) {
                if (tempScale !== 'C') {
                     result = getC(value);
                }
            } else {
                value = 0; 
            }
            return result;
        }
    );*/
	//function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
    /*gc.databind.registry.bind('widget.ti_widget_label_did.label', 'reg.ID_REG',getHex16);
    gc.databind.registry.bind('widget.ti_widget_label_nvmloc1.label', 'reg.NVM_LOC1', getHex16);
    gc.databind.registry.bind('widget.ti_widget_label_nvmloc2.label', 'reg.NVM_LOC2', getHex16);
    gc.databind.registry.bind('widget.ti_widget_label_nvmloc3.label', 'reg.NVM_LOC3', getHex16);
    gc.databind.registry.bind('widget.ti_widget_label_nvmloc4.label', 'reg.NVM_LOC4', getHex16);
	*/
	gc.databind.registry.bind('widget.ti_widget_label_did.label', 'reg.ID_REG',getHex16);
	extendAboutBox();
});

/*
 *  Boilerplate code for creating custom actions
 */

document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    regData = JSON.parse(contents);
                    if (regData) {
                        templateObj.setRegisterData(regData);
                        templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    };
    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: saved to " + result.localPath);
                    }

                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                gc.File.browseAndSave(regDataJson, 'regs.json', '*.json', function(result, errorInfo) {
                    if (errorInfo) {
                        templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                    } else if ((result) && (result.localPath)) {
                        templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: saved to " + result.localPath);
                    }
                });
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

    gc.nav.registerAction('open_log_pane', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_eventlog_view.openView();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });

});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var templateObj;
var registerModel;
var graphWidgetTempScale;


// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');
    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            console.log("Application template has been stamped.");
            templateObj.$.ti_widget_toast.hideToast();
            templateObj.$.ti_widget_eventlog_view.log("info", "Application started.");
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
			
			var graphWidgetTempScale = gc.databind.registry.getBinding('widget.ti_widget_temperaturegraph.temperatureScale'); 
			
            registerModel.addEventListener("usbHidModelReceivingData", function(event) {
                //templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
            });
            templateObj.$.ti_widget_temperaturegraph.addEventListener("graphReady", function(event) {
                console.log("graph ready event received");
            });

            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId

            //templateObj.$.ti_widget_menubar.productName = templateObj.$.ti_widget_register_page.deviceName;

            templateObj.updateTempScale = function() {
                var origScale = tempScale;
                if (ti_widget_radio_group_scale.selectedText.toLowerCase() === "farenheit") {
                    tempScale = "F";
                } else {
                    tempScale = "C";
                }
                if (origScale !== tempScale) {
                    templateObj.$.ti_widget_temperaturegraph.reset();
                    //templateObj.$.ti_widget_temperaturegraph.temperatureScale = tempScale;
					graphWidgetTempScale.setValue(tempScale);
                    var isCelsius = (tempScale === "C");
                    templateObj.$.ti_widget_spinner_high_limit.value = convertTemp(templateObj.$.ti_widget_spinner_high_limit.value, isCelsius);
                    templateObj.$.ti_widget_spinner_low_limit.value = convertTemp(templateObj.$.ti_widget_spinner_low_limit.value, isCelsius);
                    templateObj.$.ti_widget_spinner_offset_temp.value = convertTemp(templateObj.$.ti_widget_spinner_offset_temp.value, isCelsius);
                    //templateObj.$.ti_widget_label_temp.label = convertTemp(templateObj.$.ti_widget_label_temp.label, isCelsius);
                }

            };

            templateObj.updateTempScale();

            templateObj.$.ti_widget_radio_group_scale.addEventListener("changed", function(event) {
                templateObj.updateTempScale();
            });
			
			templateObj.updateMode = function() {
                    var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.CONV_MODE')
					if (templateObj.$.ti_widget_radio_group_mode.selectedText.toLowerCase() === "continuous") {
							bind.setValue(0); 
							templateObj.$.ti_widget_button_sendoneshot.disabled = true;
					} else {
                            bind.setValue(1);
							templateObj.$.ti_widget_button_sendoneshot.disabled = false;
					}
				};
 //           templateObj.updateMode();
                                                
			templateObj.$.ti_widget_button_sendoneshot.addEventListener("click", function(event) {
					var bind = gc.databind.registry.getBinding('reg.CONFIG_REG.CONV_MODE');
					         bind.setValue(1);
					         bind.setValue(3);
            });
                                                
            
        //            In the attached function for the graph, add:
        templateObj.$.ti_widget_vtabcontainer.addEventListener("selected_tab_changed",function(event){
             templateObj.$.ti_widget_temperaturegraph.asyncInitOptions(true);
             templateObj.async(templateObj.$.ti_widget_temperaturegraph.onResize,1);
        });


            templateObj.$.ti_widget_button_clearchart.addEventListener("click", function(event) {
                templateObj.$.ti_widget_temperaturegraph.reset();
            });

            templateObj.$.ti_widget_button_savechart.addEventListener("click", function(event) {
                var result = "Sample#,Temperature [" + tempScale + "],Time, Date\n";
                var strData = templateObj.$.ti_widget_temperaturegraph.getDataAsJsonStr();
                var strObj = JSON.parse(strData);
                for (var i = 0; i < strObj.length; i++) {
                    var dataPoint = strObj[i];
                    var timestamp = convertTimestamp(dataPoint[0]);
                    var temp = dataPoint[1];
                    result = result + i + "," +temp + "," + timestamp + "\n"
                }
                saveText("tempChart.csv", result);
            });

            // open the eventlog viewer when the user clicks on the book icon in the status bar:
            templateObj.$.ti_widget_eventlog_view.closeView(); // ensure that view states are in sync with application
            templateObj.$.ti_widget_statusbar.addEventListener("appiconclicked", function(event) {
                templateObj.$.ti_widget_eventlog_view.toggleView();

            });

			/*
			templateObj.$.ti_widget_button_sendoneshot.addEventListener("click", function(event) {
                var bind = gc.databind.registry.getBinding('reg.config_reg.conv_mode'); 
			// hopefully this is clear, in quotes you would enter the same thing as what is used for binding specification, including model name. 
			bind.setValue(3); 
            });*/

            // demo of handling when the user clicks the Write Register button
            templateObj.$.ti_widget_register_page.addEventListener("write_reg", function(event) {
                var detailsObj = event.detail;
                templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
            });
           
            templateObj.$.ti_widget_button_fw.addEventListener("click", function(event) {
                templateObj.$.ti_widget_label13_fw.label = templateObj.$.my_model.getCodec().version;
            });
            
            templateObj.$.ti_widget_button_soft_reset.addEventListener("click", function(event) {
                var u2a = templateObj.$.my_model.getCodec();
                var registerWrite = {
                            size : 0,
                            addr : 0x06,
                            deviceAddr : 0x00
                };
               
               u2a.writeValue(registerWrite,0);

            }); 
            
            
            

            // other event handlers added programmatically:
            // other event handlers added programmatically:
            templateObj.$.ti_widget_register_page.addEventListener("read_reg", function(event) {
                var detailsObj = event.detail;
                templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("read_regs", function(event) {
                var eventInfo = event.detail;
                templateObj.logMsgAndNotify(eventInfo.statusType, eventInfo.statusMsg, false, true);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("update_mode_changed", function(event) {
                var detailsObj = event.detail;
                templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
            }.bind(this));

            templateObj.$.ti_widget_register_page.addEventListener("auto_read_config_changed", function(event) {
                var detailsObj = event.detail;
                templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
            }.bind(this));


            var usbHidService = gc.services['ti-service-usbhid'];
            if (usbHidService) {
                usbHidService.addEventListener('usbHidError', function(event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify("error", detailsObj.info + ": " + detailsObj.error, false, true);
                    gc.connectionManager.doDisconnect();
                })
            }



            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "oneui-register-data",
                    data: []
                }
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var i = 0; i < regBlocks.length; i++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[i].registers.length; r++) {
                        var reg = regBlocks[i].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        var item = {
                            idx: r,
                            id: regName,
                            value: reg.value
                        }
                        registersData.push(item);
                    }
                    regData.data.push(registersData);
                }
                var strJson = JSON.stringify(regData);
                return strJson;
            };

            templateObj.setRegisterData = function(strJson) {
                var regData = JSON.parse(strJson);
                var ok = true;
                var regBlocks = templateObj.$.ti_widget_register_page.regBlocks;
                for (var b = 0; b < regBlocks.length; b++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[b].registers.length; r++) {
                        var reg = regBlocks[b].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        item = regData.data[b][r];
                        if (item.id === regName) {
                            reg.value = item.value;
                            templateObj.$.ti_widget_register_page.writeValueToTargetRegister(intValue, null, b, r);
                        } else {
                            var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                        }
                    }

                }
            };

            templateObj.$.ti_widget_vtabcontainer.addEventListener("before_selected_tab_changed", function(event) {
                templateObj.$.ti_widget_temperaturegraph.ignoreNextResizeEvent = true;
            });
            templateObj.showToastMsg = function(msgType, msg, durationInSeconds) {
                templateObj.$.ti_widget_toast.hideToast();
                templateObj.$.ti_widget_toast.message = msg;
                switch (msgType.toLowerCase()) {
                    case "error":
                        templateObj.$.ti_widget_toast.iconName = "error";
                        templateObj.$.ti_widget_toast.backgroundColor = "red";
                        break;
                    case "warning":
                        templateObj.$.ti_widget_toast.iconName = "warning";
                        templateObj.$.ti_widget_toast.backgroundColor = "orange";
                        break;
                    default:
                        templateObj.$.ti_widget_toast.iconName = "check";
                        templateObj.$.ti_widget_toast.backgroundColor = "teal";
                        break;
                }
                templateObj.$.ti_widget_toast.duration = +durationInSeconds;
                templateObj.$.ti_widget_toast.showToast();
                console.log("toast msg [" + msgType + "]: " + msg);
            };

            templateObj.logMsgAndNotify = function(msgType, msg, showInStatusBar, showInToast) {
                var iconName = templateObj.$.ti_widget_eventlog_view.iconForItem(msgType);
                if (showInStatusBar) {
                    var statusBar = gc.services['ti-widget-statusbar'];
                    if (statusBar) {
                        statusBar.statusString3 = msg;
                        statusBar.setIcon(iconName);
                    }
                }
                var duration = 3;
                switch (msgType) {
                    case "warning":
                        duration = 4;
                        break;
                    case "error":
                        duration = 6;
                        break;
                    default:
                        duration = 3;
                        break;
                }
                if (showInToast) {
                    templateObj.showToastMsg(msgType, msg, duration);
                }
                templateObj.$.ti_widget_eventlog_view.log(msgType, msg);
            };
            Polymer.updateStyles();
        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}